/**
 * 기본 컨트롤러 모든 컨트롤러는 이 컨트롤러를 상속받는다.
 * 기본적인 체크는 여기서 이루어진다.
 * 
 * @author Antop
 */
package kr.co.nervteam.api.controller;

import java.util.Date;
import java.util.Enumeration;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import kr.co.nervteam.api.vo.InVo;
import kr.co.nervteam.api.vo.OutVo;

import org.apache.log4j.Logger;
import org.springframework.web.bind.RequestUtils;
import org.springframework.web.bind.ServletRequestBindingException;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.AbstractController;

@SuppressWarnings("deprecation")
abstract public class BaseController extends AbstractController {

	protected Logger log = Logger.getLogger(getClass());
	private String type = Type.xml.toString();

	@SuppressWarnings("unchecked")
	@Override
	protected ModelAndView handleRequestInternal(HttpServletRequest request,
			HttpServletResponse response) throws Exception {

		// 입력 Value Object
		InVo inVo = new InVo();
		// 요청 시간
		inVo.setDateTime(new Date());
		// URL
		inVo.setUri(request.getRequestURL().toString());
		// 파라미터
		Enumeration<String> keys = request.getParameterNames();
		String key;
		String value;
		while (keys.hasMoreElements()) {
			key = keys.nextElement();
			value = request.getParameter(key);
			inVo.addParam(key, value); // set
		}
		// 출력 Value Object
		OutVo outVo = new OutVo(inVo);

		try {
			// 출력 타입 검사
			try {
				type = Type.valueOf(RequestUtils.getStringParameter(request, "t")).toString();
			} catch (Exception e) {

			}

			try {
				// 페이지 번호
				Integer page = RequestUtils.getIntParameter(request, "page");
				if (page != null && page > 0) {
					inVo.setPage(page);
				}
			} catch (ServletRequestBindingException e) {

			}

			try {
				// 줄 수
				Integer row = RequestUtils.getIntParameter(request, "row");
				if (row != null && row > 0) {
					inVo.setRow(row);
				}
			} catch (ServletRequestBindingException e) {

			}

			// 처리
			performTask(outVo);

		} catch (Exception e) {
			e.printStackTrace();
			outVo.setCode("500");
			outVo.setMsg(e.getMessage());
		}

		// 처리 완료 시간 기록
		outVo.setComplate();

		// view 설정
		String className = getClass().getSimpleName().toString().toLowerCase();
		String viewName = className.substring(0, className.length() - 10);

		ModelAndView mav = new ModelAndView();
		mav.setViewName(getForwardUrl(viewName));
		// outVo 등록
		mav.addObject("outVo", outVo);

		return mav;
	}

	/**
	 * 조회하게 되는 메소드
	 * @param outVo 출력 Value Object
	 * @throws Exception
	 */
	abstract public void performTask(OutVo outVo) throws Exception;
	/**
	 * 포워드 url 생성
	 * 
	 * @param forward
	 * @return
	 */
	protected String getForwardUrl(String forward) {
		return getType() + "/" + forward;
	}

	/**
	 * 값이 비어있는지 검사
	 * 
	 * @param s
	 * @return
	 */
	protected boolean isEmpty(String s) {
		if (s == null) {
			return true;
		}
		if (s.trim().length() == 0) {
			return true;
		}
		return false;
	}
	/**
	 * String -> Integer
	 * @param s
	 * @return
	 */
	protected Integer parseInt(String s) {
		if (s == null || s.trim().length() == 0) {
			return null;
		}

		return Integer.parseInt(s);
	}

	public String getType() {
		return type;
	}

	private enum Type {
		json, xml
	}
}
